package ACL2::Test::ManejDetalleDiff;

#
# Specialized XML proof SemanticDiff handler
# Copyright (C) 2008 Antonio García Domínguez
# $Id$
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;

=head2 new

Uso: Constructor de la clase manejadora para XML::SemanticDiff.

=cut

sub new {       
    my ($proto, %args) = @_;
    my $class = ref($proto) || $proto;
    my $self  = \%args;
    bless ($self, $class);
    return $self;
}

=head2 rogue_element

Uso: Manejador de elementos que estE<aacute>n en el segundo documento pero no en el primero.

=cut

sub rogue_element {
    my $self = shift;
    my ($element, $properties) = @_;
    my ($element_name, $parent) = _parent_and_name($element);
    my $info = {context => $parent,
                message => "Elemento inesperado '$element_name' en elemento '$parent'.",
		event => "rogue_element",
		element_name => $element_name};
    
    if ($self->{keeplinenums}) {
        $info->{startline} = $properties->{TagStart};
        $info->{endline}   = $properties->{TagEnd};
    }
 
    if ($self->{keepdata}) {
        $info->{new_value} = $properties->{CData};
    }    
    return $info;
}

=head2 missing_element

Uso: Manejador de elementos que estE<aacute>n en el primer documento pero no en el segundo.

=cut

sub missing_element {
    my $self = shift;
    my ($element, $properties) = @_;
    my ($element_name, $parent) = _parent_and_name($element);
    my $info = {context => $parent,
                message => "Elemento hijo '$element_name' falta en el elemento '$parent'.",
		event => "missing_element",
		element_name => $element_name};
                 
    if ($self->{keeplinenums}) {
        $info->{startline} = $properties->{TagStart};
        $info->{endline}   = $properties->{TagEnd};
    }
    if ($self->{keepdata}) {
        $info->{old_value} = $properties->{CData};
    }
    return $info;
}

=head2 element_value

Uso: Manejador de diferencias de contenidos entre las dos versiones del elemento.

=cut

sub element_value {
    my $self = shift;
    my ($element, $new_properties, $old_properties) = @_;
    my ($element_name, $parent) = _parent_and_name($element);

    my $info = {context => $element,
                message => "Diferencias en contenido textual en el elemento '$element_name'.",
		event => "element_value",
		element_name => $element_name};
                       
    if ($self->{keeplinenums}) {
        $info->{startline} = $new_properties->{TagStart};
        $info->{endline}   = $new_properties->{TagEnd};
    }
                  
    if ($self->{keepdata}) {
        $info->{old_value} = $old_properties->{CData};
        $info->{new_value} = $new_properties->{CData};
    }

    return $info;
}

=head2 rogue_attribute

Uso: Manejador de falta en el primer documento del atributo del segundo.

=cut

sub rogue_attribute {
    my $self = shift;    
    my ($attr, $element, $properties) = @_;
    my ($element_name, $parent) = _parent_and_name($element);
    my $info = {context  => $element,
                message  => "Atributo '$attr' inesperado en el elemento '$element_name'.",
		event => "rogue_attribute",
		element_name => $element_name,
		attribute_name => $attr};
        
    if ($self->{keeplinenums}) {
        $info->{startline} = $properties->{TagStart};
        $info->{endline}   = $properties->{TagEnd};
    }

    if ($self->{keepdata}) {
        $info->{new_value} = $properties->{Attributes}->{$attr};
    }
    return $info;
}

=head2 missing_attribute

Uso: Manejador de falta en el segundo documento del atributo del primero.

=cut

sub missing_attribute {
    my $self = shift;
    my ($attr, $element, $new_properties, $old_properties) = @_;
    my ($element_name, $parent) = _parent_and_name($element);
    my $info = {context  => $element,
                message  => "Falta el atributo '$attr' del elemento '$element_name'.",
		event => "missing_attribute",
		element_name => $element_name,
		attribute_name => $attr};
         
    if ($self->{keeplinenums}) {
        $info->{startline} = $new_properties->{TagStart};
        $info->{endline}   = $new_properties->{TagEnd};
    }

    if ($self->{keepdata}) {
        $info->{old_value} = $old_properties->{Attributes}->{$attr};
    }
    return $info;
}

=head2 attribute_value

Uso: Manejador de diferencias en los valores de las dos versiones del atributo.

=cut

sub attribute_value {
    my $self = shift;
    my ($attr, $element, $new_properties, $old_properties) = @_;
    my ($element_name, $parent) = _parent_and_name($element);
    my $info = {context  => $element,
                message  => "El atributo '$attr' tiene un valor distinto en el elemento '$element_name'.",
		event => "attribute_value",
		element_name => $element_name,
		attribute_name => $attr};
                  
    if ($self->{keeplinenums}) {
        $info->{startline} = $new_properties->{TagStart};
        $info->{endline}   = $new_properties->{TagEnd};
    }
        
    if ($self->{keepdata}) {
        $info->{old_value} = $old_properties->{Attributes}->{$attr};
        $info->{new_value} = $new_properties->{Attributes}->{$attr};
    }
    return $info;
}

=head2 namespace_uri

Uso: Manejador de diferencias en los nombres de espacio de las dos versiones de un elemento.

=cut

sub namespace_uri {
    my $self = shift;
    my ($element, $new_properties, $old_properties) = @_;
    my ($element_name, $parent) = _parent_and_name($element);
    my $info = {context  => $element,
                message  => "Elemento '$element_name' en un nombre de espacios distinto.",
		event => "namespace_uri",
		element_name => $element_name};
            
    if ($self->{keeplinenums}) {
        $info->{startline} = $new_properties->{TagStart};
        $info->{endline}   = $new_properties->{TagEnd};
    }
                            
    if ($self->{keepdata}) {
        $info->{old_value} = $old_properties->{NamspaceURI};
        $info->{new_value} = $new_properties->{NamspaceURI};
    }
    return $info;
}

sub _parent_and_name {
    my $element = shift;
    my @steps = split('/', $element);   
    my $element_name = pop (@steps);
    my $parent = join '/', @steps;
    $element_name =~ s/\[\d+\]$//;
    return ($element_name, $parent);
}

1;
__END__

=head1 NAME

ACL2::Test::ManejDetalleDiff - Manejador en detalle de diferencias semE<aacute>nticas en XML

=head1 SYNOPSIS

  use XML::SemanticDiff;
  my $diff = XML::SemanticDiff->new(diffhandler => new ACL2::Test::ManejDetalleDiff);
        
  foreach my $change ($diff->compare($file, $file2)) {
      print "$change->{event} in context $change->{context}\n";
  }

=head1 DESCRIPTION

Este mE<oacute>dulo es una versiE<oacute>n revisada del mE<oacute>dulo
XML::SemanticDiff:BasicHandler original, registrando de forma fE<aacute>cil de
manipular y estable los eventos generados durante la comparaciE<oacute>n.

=head1 AUTHOR

Antonio GarcE<iacute>a DomE<iacute>nguez <lt>nyoescape at gmail.com<gt>

=head1 COPYRIGHT
                  
La mayor parte es Copyright (c) 2000 Kip Hampton, con porciones Copyright (c) 2007-2008 Antonio GarcE<iacute>a DomE<iacute>nguez.

=head1 SEE ALSO

XML::SemanticDiff, XML::SemanticDiff::BasicHandler

=cut
